<?php
// 隐私号码保护
namespace PrivateNumberSDK;

use GuzzleHttp\Exception\RequestException;

class PrivateNumber
{
    // 网站的appkey
    private $appkey = "";
    // appkey 对应的秘钥
    private $secret = "";
    public $apiurl  = "http://api.mixcom.cn";

    /**
     * 设置appkey
     * @param appkey
     */
    public function setAppkey($appkey)
    {
        $this->appkey = $appkey;
    }

    /**
     * 设置秘钥
     * @param secret
     */
    public function setSecret($secret)
    {
        $this->secret = $secret;
    }

    /**
     * 获取服务器的时间
     * @return
     */
    public function getServerTime()
    {
        $url  = $this->apiurl . "/virtual/getServerTime";
        $json = $this->get($url);
        return $json;
    }

    /**
     * 获取可用的axb号码
     * region_code 区号，如深圳区号为 0755， 通过这个参数查询指定地区的axb号码， 可选
     * 获取号码的优先级: 绑定数量少优先 > 绑定总数少优先
     * @return
     */
    public function getAxbNumber($arr = [])
    {
        $url    = $this->apiurl . "/virtual/getaxbnumber";
        $params = [
            'region_code' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * axb号码绑定
     * number 虚拟小号，必选
     * aparty 呼叫方的号码，必选
     * bparty 被叫方的号码，必选
     * recording 是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能，可选
     * recordhinttone 录音的提示音，可选
     * maxduration 最大通话时长，单位为分钟，到了时间会自动挂断，可选
     * lastminvoice 通话最后一分钟的提示音, 当最大通话时长不为0时有效，可选
     * callerhinttone  A拨打X通话前等待音，可选
     * calleeHintTone  B拨打X通话前等待音，可选
     * direction 呼叫方向， 可选值0, 1, 2. 0代表可双向呼叫，可选
     * enddate 解绑时间，时间戳格式，可选，0代表不设置解绑时间
     * @return
     */
    public function axbBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/axbBind";
        $params = [
            'number'         => '',
            'aparty'         => '',
            'bparty'         => '',
            'recording'      => 0,
            'recordhinttone' => '',
            'maxduration'    => 0,
            'lastminvoice'   => '',
            'callerhinttone' => '',
            'calleeHintTone' => '',
            'direction'      => 0,
            'enddate'        => 0,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * axb号码解绑
     * number 虚拟小号，必选
     * aparty 呼叫方的号码，必选
     * bparty 被叫方的号码，必选
     * @return
     */
    public function unAxbBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/unAxbBind";
        $params = [
            'number' => '',
            'aparty' => '',
            'bparty' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * axb号码自动绑定
     * aparty 呼叫方的号码，必选
     * bparty 被叫方的号码，必选
     * recording 是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能，可选
     * recordhinttone 录音的提示音，可选
     * maxduration 最大通话时长，单位为分钟，到了时间会自动挂断，可选
     * lastminvoice 通话最后一分钟的提示音, 当最大通话时长不为0时有效，可选
     * callerhinttone  A拨打X通话前等待音，可选
     * calleeHintTone  B拨打X通话前等待音，可选
     * direction 呼叫方向， 可选值0, 1, 2. 0代表可双向呼叫，可选
     * enddate 解绑时间，时间戳格式，可选，0代表不设置解绑时间
     * @return
     */
    public function axbAutoBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/axbAutoBind";
        $params = [
            'aparty'         => '',
            'bparty'         => '',
            'region_code'    => '',
            'defregioncode'  => '',
            'recording'      => 0,
            'recordhinttone' => '',
            'maxduration'    => 0,
            'lastminvoice'   => '',
            'callerhinttone' => '',
            'calleeHintTone' => '',
            'direction'      => 0,
            'enddate'        => 0,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 获取可用的ax号码
     * region_code 区号，如深圳区号为 0755， 通过这个参数查询指定地区的ax号码， 可选
     * @return
     */
    public function getAxNumber($arr = [])
    {
        $url    = $this->apiurl . "/virtual/getAxNumber";
        $params = [
            'region_code' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * ax号码绑定
     * number 虚拟小号，必选
     * aparty 呼叫方的号码，必选
     * recording  是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能，可选
     * recordhinttone 录音前播放的提示音，可选
     * display_call 是否开启号码透传。默认为0，不开启。1为开启，开启后可能会导致部分通话无法接通，可选
     * callerhinttone  A拨打X通话前等待音，可选
     * calleeHintTone  B拨打X通话前等待音，可选
     * enddate 解绑时间，时间戳格式，可选，0代表不设置解绑时间
     * @return
     */
    public function axBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/axBind";
        $params = [
            'number'         => '',
            'aparty'         => '',
            'recording'      => 0,
            'recordhinttone' => '',
            'display_call'   => 0,
            'callerhinttone' => '',
            'calleehinttone' => '',
            'enddate'        => 0,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * ax号码解绑
     * number 虚拟小号，必选
     * aparty 呼叫方的号码，必选
     * @return
     */
    public function unAxBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/unAxBind";
        $params = [
            'number' => '',
            'aparty' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * AX绑定呼出
     * number 虚拟小号，必选
     * aparty 呼叫方的号码，必选
     * bparty 被叫方的号码，必选
     * @return
     */
    public function axDialNumber($arr = [])
    {
        $url    = $this->apiurl . "/virtual/axdialnumber";
        $params = [
            'number' => '',
            'aparty' => '',
            'bparty' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * ax号码自动绑定
     * aparty 呼叫方的号码，必选
     * region_code 需要的虚拟小号的区号, 可选
     * display_call 是否开启号码透传。默认为0，不开启。1为开启，开启后可能会导致部分通话无法接通，可选
     * recording  是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能，可选
     * recordhinttone 录音前播放的提示音，可选
     * callerhinttone  A拨打X通话前等待音，可选
     * calleeHintTone  B拨打X通话前等待音，可选
     * enddate 解绑时间，时间戳格式，可选，0代表不设置解绑时间
     * @return
     */
    public function axAutoBind($arr = [])
    {
        $url    = $this->apiurl . "/virtual/axAutoBind";
        $params = [
            'aparty'         => '',
            'region_code'    => '',
            'defregioncode'  => '',
            'display_call'   => 0,
            'recording'      => 0,
            'recordhinttone' => '',
            'callerhinttone' => '',
            'calleehinttone' => '',
            'enddate'        => 0,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 挂断指定通话
     * subscriptionid 该通话的绑定关系id
     * @return
     */
    public function callStop($arr)
    {
        $url    = $this->apiurl . "/virtual/callStop";
        $params = [
            'subscriptionid' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 获取小号的个数
     * @return
     */
    public function countNumber()
    {
        $url    = $this->apiurl . "/virtual/countNumber";
        $params = [];
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 查询所有的号码
     * page 需要查询的页数，每页100条数据
     * @return
     */
    public function queryAllNumber($arr = [])
    {
        $url    = $this->apiurl . "/virtual/queryAllNumber";
        $params = [
            'page' => 1,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 查询号码的绑定关系
     * number 虚拟小号，可选
     * aparty 呼叫方的号码，可选
     * bparty 被叫方的号码，可选
     * @return
     */
    public function queryBindRelation($arr = [])
    {
        $url    = $this->apiurl . "/virtual/querybindrelation";
        $params = [
            'number' => '',
            'aparty' => '',
            'bparty' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 查询号码的绑定关系日志
     * number 虚拟小号，可选
     * aparty 呼叫方的号码，可选
     * bparty 被叫方的号码，可选
     * timestart 开始时间，可选
     * timeend 结束时间，可选
     * page 页数，可选
     * @return
     */
    public function queryRelationLog($arr = [])
    {
        $url    = $this->apiurl . "/virtual/queryrelationlog";
        $params = [
            'number'    => '',
            'aparty'    => '',
            'bparty'    => '',
            'timestart' => 0,
            'timeend'   => 0,
            'page'      => 1,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 查询通话记录
     * number 虚拟小号，可选
     * aparty 呼叫方的号码，可选
     * bparty 被叫方的号码，可选
     * timestart 开始时间，可选
     * timeend 结束时间，可选
     * page 页数，可选
     * @return
     */
    public function queryCallRecord($arr = [])
    {
        $url    = $this->apiurl . "/virtual/querycallrecord";
        $params = [
            'number'    => '',
            'aparty'    => '',
            'bparty'    => '',
            'timestart' => 0,
            'timeend'   => 0,
            'page'      => 1,
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    /**
     * 返回录音的下载链接(在录音未删除之前有效)
     * number 虚拟小号，必选
     * callIdentifier 通话录音id，必选
     */
    public function getRecordUrl($arr = [])
    {
        $url    = $this->apiurl . "/virtual/getrecordurl";
        $params = [
            'number'         => '',
            'callIdentifier' => '',
        ];
        $params = array_merge($params, $arr);
        $json   = $this->post($url, $params);
        return $json;
    }

    private function get($url)
    {
        $client = new \GuzzleHttp\Client();
        try {
            $response = $client->get($url);
            $status   = $response->getStatusCode();
            if ($status == '200') {
                $body = $response->getBody();
                $data = null;
                while (!$body->eof()) {
                    $data .= $body->read(1024);
                }
                return json_decode($data);
            }
        } catch (RequestException $e) {
            // 请求异常，可能是网络异常
        }
    }

    private function post($url, $params = [])
    {
        $params['time']   = time();
        $params['appkey'] = $this->appkey;
        $params['sign']   = $this->getSign($params);

        $client = new \GuzzleHttp\Client();
        try {
            $response = $client->post($url, [
                'body' => $params,
            ]);
            $status = $response->getStatusCode();
            if ($status == '200') {
                $body = $response->getBody();
                $data = null;
                while (!$body->eof()) {
                    $data .= $body->read(1024);
                }
                return json_decode($data);
            }
        } catch (RequestException $e) {
            // 请求异常，可能是网络异常
        }
    }

    private function getSign($params)
    {
        // key转换为小写
        $params = array_change_key_case($params, CASE_LOWER);
        // 排除不需要加密的参数
        foreach ($params as $key => $val) {
            if ($key == 'c' || $key == 'a' || $key == 'sign') {
                // 从数组中移除
                unset($params[$key]);
            }
            if ($val === '') {
                // 空值从数组中移除
                unset($params[$key]);
            }
        }
        // 对key进行排序, 按名称升序
        ksort($params);
        // 将数据连接，进行加密
        $data = '';
        foreach ($params as $key => $val) {
            $data .= $val;
        }
        $data .= $this->secret;
        $sign = strtolower(sha1($data . sha1($this->secret)));
        return $sign;
    }
}
